"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const types_1 = require("./types");
const token_math_1 = require("@melonproject/token-math");
exports.createOrderbook = (options, orders) => ({
    quote: options.pair.quote,
    base: options.pair.quote,
    network: options.network,
    asks: (orders && orders.asks) || [],
    bids: (orders && orders.bids) || [],
});
exports.sortOrders = (a, b) => {
    const priceA = token_math_1.toAtomic(a.trade);
    const priceB = token_math_1.toAtomic(b.trade);
    const difference = parseFloat(token_math_1.subtract(priceB, priceA).toString());
    // Sort by volumes if prices are identical.
    if (difference === 0) {
        const quantityA = a.trade.base.quantity;
        const quantityB = b.trade.base.quantity;
        return parseFloat(token_math_1.subtract(quantityB, quantityA).toString());
    }
    return difference;
};
exports.reduceOrderVolumes = (carry, order, index) => {
    const token = R.path(['trade', 'base', 'token'], order);
    const current = R.path(['trade', 'base', 'quantity'], order);
    const previous = R.path([index - 1, 'cummulative'], carry);
    const cummulative = token_math_1.add(token_math_1.createQuantity(token, current), previous || token_math_1.createQuantity(token, 0));
    return (carry || []).concat([
        Object.assign({}, order, { cummulative }),
    ]);
};
exports.isBidOrder = R.propEq('type', types_1.OrderType.BID);
exports.isAskOrder = R.propEq('type', types_1.OrderType.ASK);
exports.aggregateOrders = (orders) => {
    const asks = orders.filter(exports.isAskOrder);
    const bids = orders.filter(exports.isBidOrder);
    return {
        asks: asks.reduce(exports.reduceOrderVolumes, []),
        bids: bids.reduce(exports.reduceOrderVolumes, []),
    };
};
exports.reduceOrderEvents = (carry, current) => {
    if (current.event === types_1.NormalizedMessageType.SNAPSHOT) {
        const snapshot = current;
        const bids = snapshot.orders.filter(order => order.type === types_1.OrderType.BID);
        const asks = snapshot.orders.filter(order => order.type === types_1.OrderType.ASK);
        const { exchange } = current;
        return {
            bids: carry.bids
                .filter(item => item.exchange !== exchange)
                .concat(bids)
                .sort(exports.sortOrders)
                .reduce(exports.reduceOrderVolumes, []),
            asks: carry.asks
                .filter(item => item.exchange !== exchange)
                .concat(asks)
                .sort(exports.sortOrders)
                .reduce(exports.reduceOrderVolumes, []),
        };
    }
    if (current.event === types_1.NormalizedMessageType.SET) {
        const add = current;
        const { id, order } = add;
        return {
            asks: order.type === types_1.OrderType.ASK
                ? carry.asks
                    .filter(item => item.id !== id)
                    .concat([order])
                    .sort(exports.sortOrders)
                    .reduce(exports.reduceOrderVolumes, [])
                : carry.asks,
            bids: order.type === types_1.OrderType.BID
                ? carry.bids
                    .filter(item => item.id !== id)
                    .concat([order])
                    .sort(exports.sortOrders)
                    .reduce(exports.reduceOrderVolumes, [])
                : carry.bids,
        };
    }
    if (current.event === types_1.NormalizedMessageType.REMOVE) {
        const remove = current;
        const { id } = remove;
        return {
            asks: carry.asks
                .filter(order => order.id !== id)
                .reduce(exports.reduceOrderVolumes, []),
            bids: carry.bids
                .filter(order => order.id !== id)
                .reduce(exports.reduceOrderVolumes, []),
        };
    }
    return carry;
};
