"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const R = __importStar(require("ramda"));
const types_1 = require("../../types");
const token_math_1 = require("@melonproject/token-math");
const getCurrenciesHttpUrl = (options) => {
    switch (options.network) {
        case types_1.Network.MAINNET:
            return 'https://api.kyber.network/currencies';
        default:
            throw new Error('Kyber only supports the MAINNET network.');
    }
};
const getRateHttpUrl = (options, type, currency, interval) => {
    if (options.network !== types_1.Network.MAINNET) {
        throw new Error('Kyber only supports the MAINNET network.');
    }
    const prefix = `https://api.kyber.network/${type}_rate`;
    const splits = R.splitEvery(5, interval);
    const suffix = splits
        .map(split => {
        const quantities = split
            .map(qty => {
            return `qty=${qty}`;
        })
            .join('&');
        return `id=${currency}&${quantities}`;
    })
        .join('&');
    return `${prefix}?${suffix}`;
};
const formatResponse = (options, type, response) => {
    if (response.error) {
        throw new Error(`Error trying to fetch Kyber ${type} rates.`);
    }
    const volumeKey = type === types_1.OrderType.ASK ? 'src_qty' : 'dst_qty';
    const priceKey = type === types_1.OrderType.ASK ? 'dst_qty' : 'src_qty';
    const groups = response.data.map(current => {
        return Object.keys(current[volumeKey]).map(index => {
            const volume = current[volumeKey][index];
            const price = current[priceKey][index];
            const oid = Buffer.from(`${types_1.Exchange.KYBER_NETWORK}:${volume}`).toString('base64');
            const trade = token_math_1.createPrice(token_math_1.createQuantity(options.pair.base, volume), token_math_1.createQuantity(options.pair.quote, price));
            return {
                id: oid,
                type,
                exchange: types_1.Exchange.KYBER_NETWORK,
                trade,
            };
        });
    });
    return [].concat(...groups);
};
exports.fetchCurrencies = (options) => __awaiter(this, void 0, void 0, function* () {
    const url = getCurrenciesHttpUrl(options);
    const response = yield axios_1.default
        .get(url)
        .then(result => result.data);
    if (response.error) {
        throw new Error(response.error);
    }
    return response.data;
});
exports.fetchRates = (options, currencies) => __awaiter(this, void 0, void 0, function* () {
    const base = options.pair.base.symbol;
    const currency = currencies.find(R.propEq('symbol', base));
    if (!currency) {
        throw new Error(`The ${base} token is not supported.`);
    }
    const quantities = options.quantities || [1, 10, 100, 1000];
    const buyUrl = getRateHttpUrl(options, 'buy', currency.id, quantities);
    const sellUrl = getRateHttpUrl(options, 'sell', currency.id, quantities);
    const sellRequest = axios_1.default.get(sellUrl).then(result => result.data);
    const buyRequest = axios_1.default.get(buyUrl).then(result => result.data);
    const [buyResponse, sellResponse] = yield Promise.all([
        buyRequest,
        sellRequest,
    ]);
    return [].concat(formatResponse(options, types_1.OrderType.BID, buyResponse), formatResponse(options, types_1.OrderType.ASK, sellResponse));
});
exports.fetch = (options) => __awaiter(this, void 0, void 0, function* () {
    const currencies = yield exports.fetchCurrencies(options);
    return exports.fetchRates(options, currencies);
});
